@@ -127,9 +127,6 @@
 
 QT_BEGIN_NAMESPACE
 
-// Crashes on AMD Radeon drivers on Windows. Disable for now.
-//#define SHADER_LOADING_IN_COMMAND_THREAD
-
 using namespace Qt3DCore;
 
 namespace Qt3DRender {
@@ -251,10 +248,10 @@ Renderer::Renderer(QRenderAspect::RenderType type)
     , m_lightGathererJob(new CachingLightGatherer(&m_cache))
     , m_renderableEntityFilterJob(new CachingRenderableEntityFilter(&m_cache))
     , m_computableEntityFilterJob(new CachingComputableEntityFilter(&m_cache))
-    , m_bufferGathererJob(CreateSynchronizerJobPtr([this] { lookForDirtyBuffers(); }, JobTypes::DirtyBufferGathering))
-    , m_vaoGathererJob(CreateSynchronizerJobPtr([this] { lookForAbandonedVaos(); }, JobTypes::DirtyVaoGathering))
-    , m_textureGathererJob(CreateSynchronizerJobPtr([this] { lookForDirtyTextures(); }, JobTypes::DirtyTextureGathering))
-    , m_sendSetFenceHandlesToFrontendJob(CreateSynchronizerJobPtr([this] { sendSetFenceHandlesToFrontend(); }, JobTypes::SendSetFenceHandlesToFrontend))
+    , m_bufferGathererJob(CreateSynchronizerJobPtr([this] { lookForDirtyBuffers(); }, JobTypes::DirtyBufferGathering, 0))
+    , m_vaoGathererJob(CreateSynchronizerJobPtr([this] { lookForAbandonedVaos(); }, JobTypes::DirtyVaoGathering, 0))
+    , m_textureGathererJob(CreateSynchronizerJobPtr([this] { lookForDirtyTextures(); }, JobTypes::DirtyTextureGathering, 0))
+    , m_sendSetFenceHandlesToFrontendJob(CreateSynchronizerJobPtr([this] { sendSetFenceHandlesToFrontend(); }, JobTypes::SendSetFenceHandlesToFrontend, 0))
     , m_introspectShaderJob(CreateSynchronizerPostFramePtr([this] { reloadDirtyShaders(); },
                                                            [this] (Qt3DCore::QAspectManager *m) { sendShaderChangesToFrontend(m); },
                                                            JobTypes::DirtyShaderGathering))
@@ -534,7 +531,11 @@ void Renderer::shutdown()
     QMutexLocker lock(&m_hasBeenInitializedMutex);
 
     qCDebug(Backend) << Q_FUNC_INFO << "Requesting renderer shutdown";
-    m_running.storeRelaxed(0);
+    const bool wasRunning = m_running.testAndSetRelaxed(1, 0);
+
+    // We might have already been shutdown
+    if (!wasRunning)
+        return;
 
     // We delete any renderqueue that we may not have had time to render
     // before the surface was destroyed
@@ -593,37 +594,50 @@ void Renderer::releaseGraphicsResources()
     QOpenGLContext *context = m_submissionContext->openGLContext();
     Q_ASSERT(context);
 
-    if (context->thread() == QThread::currentThread() && context->makeCurrent(offscreenSurface)) {
+    if (context->thread() == QThread::currentThread()) {
+        QSurface *lastContextSurface = context->surface();
 
-        // Clean up the graphics context and any resources
-        const std::vector<HGLTexture> &activeTexturesHandles = m_glResourceManagers->glTextureManager()->activeHandles();
-        for (const HGLTexture &textureHandle : activeTexturesHandles) {
-            GLTexture *tex = m_glResourceManagers->glTextureManager()->data(textureHandle);
-            tex->destroy();
+        if (context->makeCurrent(offscreenSurface)) {
+            // Clean up the graphics context and any resources
+            const std::vector<HGLTexture> &activeTexturesHandles = m_glResourceManagers->glTextureManager()->activeHandles();
+            for (const HGLTexture &textureHandle : activeTexturesHandles) {
+                GLTexture *tex = m_glResourceManagers->glTextureManager()->data(textureHandle);
+                tex->destroy();
+            }
+
+            // Do the same thing with buffers
+            const std::vector<HGLBuffer> &activeBuffers = m_glResourceManagers->glBufferManager()->activeHandles();
+            for (const HGLBuffer &bufferHandle : activeBuffers) {
+                GLBuffer *buffer = m_glResourceManagers->glBufferManager()->data(bufferHandle);
+                buffer->destroy(m_submissionContext.data());
+            }
+
+            // Do the same thing with shaders
+            const QVector<GLShader *> shaders = m_glResourceManagers->glShaderManager()->takeActiveResources();
+            qDeleteAll(shaders);
+
+            // Do the same thing with VAOs
+            const std::vector<HVao> &activeVaos = m_glResourceManagers->vaoManager()->activeHandles();
+            for (const HVao &vaoHandle : activeVaos) {
+                OpenGLVertexArrayObject *vao = m_glResourceManagers->vaoManager()->data(vaoHandle);
+                vao->destroy();
+            }
+
+            m_submissionContext->releaseRenderTargets();
+
+            m_frameProfiler.reset();
+            if (m_ownedContext) {
+                context->doneCurrent();
+            } else {
+                // Leave the context in the state we found it in by restoring
+                // its last used surface. This satisfies expectations when used
+                // with QQuickWidgets that surface on current context after
+                // QQuickRenderControl cleanup is the same as prior to the
+                // cleanup. Arguably this could also be checked for in
+                // QQuickWidgetPrivate::invalidateRenderControl.
+                context->makeCurrent(lastContextSurface);
+            }
         }
-
-        // Do the same thing with buffers
-        const std::vector<HGLBuffer> &activeBuffers = m_glResourceManagers->glBufferManager()->activeHandles();
-        for (const HGLBuffer &bufferHandle : activeBuffers) {
-            GLBuffer *buffer = m_glResourceManagers->glBufferManager()->data(bufferHandle);
-            buffer->destroy(m_submissionContext.data());
-        }
-
-        // Do the same thing with shaders
-        const QVector<GLShader *> shaders = m_glResourceManagers->glShaderManager()->takeActiveResources();
-        qDeleteAll(shaders);
-
-        // Do the same thing with VAOs
-        const std::vector<HVao> &activeVaos = m_glResourceManagers->vaoManager()->activeHandles();
-        for (const HVao &vaoHandle : activeVaos) {
-            OpenGLVertexArrayObject *vao = m_glResourceManagers->vaoManager()->data(vaoHandle);
-            vao->destroy();
-        }
-
-        m_submissionContext->releaseRenderTargets();
-
-        m_frameProfiler.reset();
-        context->doneCurrent();
     } else {
         qWarning() << "Failed to make context current: OpenGL resources will not be destroyed";
     }
@@ -952,6 +966,13 @@ void Renderer::prepareCommandsSubmission(const QVector<RenderView *> &renderView
                 // RenderCommand should have a handle to the corresponding VAO for the Mesh and Shader
                 HVao vaoHandle;
 
+                // If shader was loaded this frame, skip creating VAO for the command
+                // as we have to wait for next frame to make sure command was build against valid shader
+                if (m_lastLoadedShaderIds.contains(command.m_shaderId)) {
+                    command.m_isValid = false;
+                    return;
+                }
+
                 // Create VAO or return already created instance associated with command shader/geometry
                 // (VAO is emulated if not supported)
                 createOrUpdateVAO(&command, &vaoHandle, &vao);
@@ -1306,7 +1327,6 @@ void Renderer::updateGLResources()
         }
     }
 
-#ifndef SHADER_LOADING_IN_COMMAND_THREAD
     {
         Profiling::GLTimeRecorder recorder(Profiling::ShaderUpload, activeProfiler());
         const QVector<HShader> dirtyShaderHandles = std::move(m_dirtyShaders);
@@ -1320,9 +1340,26 @@ void Renderer::updateGLResources()
 
             // Compile shader
             m_submissionContext->loadShader(shader, shaderManager, m_glResourceManagers->glShaderManager());
+
+            // Release any VAO referencing this shader. When we build VAO, we
+            // rely on the shader introspection to know the active uniforms In
+            // case the shader is reloaded, we might end up having more/less
+            // active uniforms than prior therefore we need to ensure VAO is
+            // rebuilt.
+            VAOManager *vaoManager = m_glResourceManagers->vaoManager();
+            const std::vector<HVao> activeVaos = vaoManager->activeHandles(); // copy
+            for (const HVao &vao : activeVaos) {
+                if (vao.data() && vao->key().second == shader->peerId())
+                    vaoManager->releaseResource(vao->key());
+            }
+
+            // Record shader id in vector of vectors loaded this frame
+            // Given commands need to be built against loaded shader (at next frame)
+            // we can make use of this vector to skip operations that target this
+            // shader for this frame
+            m_lastLoadedShaderIds.push_back(shader->peerId());
         }
     }
-#endif
 
     {
         Profiling::GLTimeRecorder recorder(Profiling::TextureUpload, activeProfiler());
@@ -1780,7 +1817,7 @@ bool Renderer::shouldRender() const
 {
     // Only render if something changed during the last frame, or the last frame
     // was not rendered successfully (or render-on-demand is disabled)
-    return (m_settings->renderPolicy() == QRenderSettings::Always
+    return ((m_settings && m_settings->renderPolicy() == QRenderSettings::Always)
             || m_dirtyBits.marked != 0
             || m_dirtyBits.remaining != 0
             || !m_lastFrameCorrect.loadRelaxed());
@@ -1847,6 +1884,7 @@ QVector<Qt3DCore::QAspectJobPtr> Renderer::renderBinJobs()
     // Remove previous dependencies
     m_cleanupJob->removeDependency(QWeakPointer<QAspectJob>());
 
+    const bool dirtyParametersForCurrentFrame = m_dirtyBits.marked & AbstractRenderer::ParameterDirty;
     const BackendNodeDirtySet dirtyBitsForFrame = m_dirtyBits.marked | m_dirtyBits.remaining;
     m_dirtyBits.marked = {};
     m_dirtyBits.remaining = {};
@@ -1970,6 +2008,10 @@ QVector<Qt3DCore::QAspectJobPtr> Renderer::renderBinJobs()
 
     m_dirtyBits.remaining = dirtyBitsForFrame & notCleared;
 
+    // Dirty Parameters might need 2 frames to react if the parameter references a texture
+    if (dirtyParametersForCurrentFrame)
+        m_dirtyBits.remaining |= AbstractRenderer::ParameterDirty;
+
     return renderBinJobs;
 }
 
@@ -2318,6 +2360,7 @@ void Renderer::cleanGraphicsResources()
         // We can really release the texture at this point
         m_nodesManager->shaderManager()->releaseResource(shaderCleanedUpId);
     }
+    m_lastLoadedShaderIds.clear();
 }
 
 const GraphicsApiFilterData *Renderer::contextInfo() const
