@@ -37,10 +37,10 @@
 **
 ****************************************************************************/
 
-#include "quick3dparameter_p_p.h"
+#include <QtQml/QJSValue>
+#include <QtQml/QJSValueIterator>
 
-#include <QJSValue>
-#include <QJSValueIterator>
+#include <Qt3DQuickRender/private/quick3dparameter_p_p.h>
 
 QT_BEGIN_NAMESPACE
 
